/***************************************************************************
                          kauditor.cpp  -  description
                             -------------------
    begin                : Tue Jul 18 12:38:24 EEST 2000
    copyright            : (C) 2000 by Marko Grnroos
    email                : magi@iki.fi
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// include files for QT
#include <qdir.h>
#include <qstrlist.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmsgbox.h>
#include <kfiledialog.h>

// application specific includes
#include "kauditor.h"
#include "kauditorview.h"
#include "kauditordoc.h"
#include "resource.h"
#include "qclipboard.h"

// User-defined
#include "trainerdlg.h"
#include "trainprogrdlg.h"
#include "plotdlg.h"
#include "statsdlg.h"

void TrainParameters::defaults () {
  	trainCycles = 10000;
  	delta0 = 0.1;
  	deltaMax = 50;
  	weightDecay = 0.99999;
  	useWeightDecay = true;
	hidden1 = 10;
	hidden2 = 5;
	hidden3 = 0;
	runs = 1;
	allInputs = true;
	allOutputs = true;
	monthInputs = true;
	equalizeGlobal = true;
	inputMonths = 1;
}

String TrainParameters::hiddenString () const return result {
	result = "-";
	if (hidden1 > 0)
		result += String(hidden1) + "-";
	if (hidden2 > 0)
		result += String(hidden2) + "-";
	if (hidden3 > 0)
		result += String(hidden3) + "-";
}


KAuditorApp::KAuditorApp()
{
  config=kapp->getConfig();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initMenuBar();
  initToolBar();
  initStatusBar();
  initKeyAccel();
  initDocument();
  initView();
	
	///////////////////////////////////////////////////////////////////
	mpParams = new TrainParameters ();
	readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable menu and toolbar items at startup
  disableCommand(ID_FILE_SAVE);
  disableCommand(ID_FILE_SAVE_AS);
  disableCommand(ID_FILE_PRINT);
 	
  disableCommand(ID_EDIT_CUT);
  disableCommand(ID_EDIT_COPY);
  disableCommand(ID_EDIT_PASTE);

	updateMenus ();
}

KAuditorApp::~KAuditorApp()
{

}

void KAuditorApp::initKeyAccel()
{
  keyAccel = new KAccel(this);
	
  // fileMenu accelerators
  keyAccel->connectItem(KAccel::New, this, SLOT(slotFileNew()));
  keyAccel->connectItem(KAccel::Open, this, SLOT(slotFileOpen()));
  keyAccel->connectItem(KAccel::Save, this, SLOT(slotFileSave()));
  keyAccel->connectItem(KAccel::Close, this, SLOT(slotFileClose()));
  keyAccel->connectItem(KAccel::Print, this, SLOT(slotFilePrint()));
  keyAccel->connectItem(KAccel::Quit, this, SLOT(slotFileQuit()));
  // editMenu accelerators
  keyAccel->connectItem(KAccel::Cut, this, SLOT(slotEditCut()));
  keyAccel->connectItem(KAccel::Copy, this, SLOT(slotEditCopy()));
  keyAccel->connectItem(KAccel::Paste, this, SLOT(slotEditPaste()));

  keyAccel->connectItem(KAccel::Help, kapp, SLOT(appHelpActivated()));
			
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_NEW, KAccel::New);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_OPEN, KAccel::Open);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_SAVE, KAccel::Save);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_CLOSE, KAccel::Close);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_PRINT, KAccel::Print);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_QUIT, KAccel::Quit);

  keyAccel->changeMenuAccel(editMenu, ID_EDIT_CUT, KAccel::Cut);
  keyAccel->changeMenuAccel(editMenu, ID_EDIT_COPY, KAccel::Copy);
  keyAccel->changeMenuAccel(editMenu, ID_EDIT_PASTE, KAccel::Paste);

  keyAccel->readSettings();	
}

void KAuditorApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR
  recentFilesMenu = new QPopupMenu();
  connect(recentFilesMenu, SIGNAL(activated(int)), SLOT(slotFileOpenRecent(int)));

  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  fileMenu = new QPopupMenu();
  //fileMenu->insertItem(kapp->getMiniIcon(), i18n("New &Window"), ID_FILE_NEW_WINDOW);
  //fileMenu->insertSeparator();
  //fileMenu->insertItem(Icon("filenew.xpm"), i18n("&New"), ID_FILE_NEW);
  fileMenu->insertItem(Icon("fileopen.xpm"), i18n("&Open..."), ID_FILE_OPEN);
  fileMenu->insertItem(i18n("Open &recent"), recentFilesMenu, ID_FILE_OPEN_RECENT);
  fileMenu->insertItem(Icon("fileopen.xpm"), i18n("Open &budget..."), ID_FILE_OPEN_BUDGET);
  fileMenu->insertItem(i18n("Open r&ecent budget"), recentFilesMenu, ID_FILE_OPEN_RECENT_BUDGET);

  //fileMenu->insertItem(i18n("&Close"), ID_FILE_CLOSE);
  //fileMenu->insertSeparator();
  //fileMenu->insertItem(Icon("filefloppy.xpm") ,i18n("&Save"), ID_FILE_SAVE);
  //fileMenu->insertItem(i18n("Save &As..."), ID_FILE_SAVE_AS);
  //fileMenu->insertSeparator();
  //fileMenu->insertItem(Icon("fileprint.xpm"), i18n("&Print..."), ID_FILE_PRINT);
  fileMenu->insertItem(Icon("fileopen.xpm"), i18n("Open &network..."), ID_FILE_OPEN_NETWORK);
  fileMenu->insertItem(Icon("filefloppy.xpm") ,i18n("&Save network"), ID_FILE_SAVE_NETWORK);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("E&xit"), ID_FILE_QUIT);
	
  ///////////////////////////////////////////////////////////////////
  // menuBar entry editMenu
  editMenu = new QPopupMenu();
  //editMenu->insertItem(Icon("editcut.xpm"), i18n("Cu&t"), ID_EDIT_CUT);
  editMenu->insertItem(Icon("editcopy.xpm"), i18n("&Copy"), ID_EDIT_COPY);
  //editMenu->insertItem(Icon("editpaste.xpm"), i18n("&Paste"), ID_EDIT_PASTE);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewMenu = new QPopupMenu();
  viewMenu->setCheckable(true);
  viewMenu->insertItem(i18n("&Toolbar"), ID_VIEW_TOOLBAR);
  viewMenu->insertItem(i18n("&Statusbar"), ID_VIEW_STATUSBAR);

  ///////////////////////////////////////////////////////////////////
  // Options menu
	optionsMenu = new QPopupMenu();
	optionsMenu->insertItem(i18n("&Training settings..."), ID_OPTIONS_TRAIN_SETTINGS);
	disableCommand(ID_OPTIONS_TRAIN_SETTINGS);
	optionsMenu->insertItem(i18n("&Save settings"), ID_OPTIONS_SAVE);

  ///////////////////////////////////////////////////////////////////
  // Data menu
	dataMenu = new QPopupMenu();
	dataMenu->insertItem(i18n("&Train neural network!"), ID_DATA_TRAIN);
	dataMenu->insertItem(i18n("&Inspect prediction statistics..."), ID_DATA_STATS);
	dataMenu->insertItem(i18n("&Plot data..."), ID_DATA_PLOT);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  QString aboutstring=kapp->appName()+" "+VERSION+"\n\n";

  helpMenu = new QPopupMenu();
  helpMenu = kapp->getHelpMenu(true, aboutstring);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  // insert your popup menus with the according menubar entries in the order
  // they will appear later from left to right
  menuBar()->insertItem(i18n("&File"), fileMenu);
  menuBar()->insertItem(i18n("&Edit"), editMenu);
  menuBar()->insertItem(i18n("&View"), viewMenu);
  menuBar()->insertItem(i18n("&Options"), optionsMenu);
  menuBar()->insertItem(i18n("&Data"), dataMenu);

  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Help"), helpMenu);

  ///////////////////////////////////////////////////////////////////
  // CONNECT THE MENU SLOTS WITH SIGNALS
  // for execution slots and statusbar messages

  connect(fileMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(fileMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

  connect(editMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(editMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

  connect(viewMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(viewMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

  connect(optionsMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(optionsMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

  connect(dataMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(dataMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
}

void KAuditorApp::initToolBar()
{

  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  //toolBar()->insertButton(Icon("filenew.xpm"), ID_FILE_NEW, true, i18n("New File"));
  toolBar()->insertButton(Icon("fileopen.xpm"), ID_FILE_OPEN, true, i18n("Open File"));
  //toolBar()->insertButton(Icon("filefloppy.xpm"), ID_FILE_SAVE, true, i18n("Save File"));
  //toolBar()->insertButton(Icon("fileprint.xpm"), ID_FILE_PRINT, true, i18n("Print"));
  toolBar()->insertSeparator();
  //toolBar()->insertButton(Icon("editcut.xpm"), ID_EDIT_CUT, true, i18n("Cut"));
  toolBar()->insertButton(Icon("editcopy.xpm"), ID_EDIT_COPY, true, i18n("Copy"));
  //toolBar()->insertButton(Icon("editpaste.xpm"), ID_EDIT_PASTE, true, i18n("Paste"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("help.xpm"), ID_HELP_CONTENTS, SIGNAL(clicked()),
  				kapp, SLOT(appHelpActivated()), true,i18n("Help"));

  ///////////////////////////////////////////////////////////////////
  // INSERT YOUR APPLICATION SPECIFIC TOOLBARS HERE WITH toolBar(n)


  ///////////////////////////////////////////////////////////////////
  // CONNECT THE TOOLBAR SLOTS WITH SIGNALS - add new created toolbars by their according number
  // connect for invoking the slot actions
  connect(toolBar(), SIGNAL(clicked(int)), SLOT(commandCallback(int)));
  // connect for the status help on holing icons pressed with the mouse button
  connect(toolBar(), SIGNAL(pressed(int)), SLOT(statusCallback(int)));

}

void KAuditorApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n(IDS_STATUS_DEFAULT), ID_STATUS_MSG);
}

void KAuditorApp::initDocument()
{
  doc = new KAuditorDoc(this);
  doc->newDocument();
}

void KAuditorApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new KAuditorView(this);
  doc->addView(view);
  setView(view);
  QString caption=kapp->getCaption();	
  setCaption(caption+": "+doc->getTitle());

}

void KAuditorApp::enableCommand(int id_)
{
  ///////////////////////////////////////////////////////////////////
  // enable menu and toolbar functions by their ID's
  menuBar()->setItemEnabled(id_, true);
  toolBar()->setItemEnabled(id_, true);
}

void KAuditorApp::disableCommand(int id_)
{
  ///////////////////////////////////////////////////////////////////
  // disable menu and toolbar functions by their ID's
  menuBar()->setItemEnabled(id_, false);
  toolBar()->setItemEnabled(id_, false);
}

void KAuditorApp::addRecentFile(const QString &file)
{
  if(recentFiles.find(file) == -1)
  {
    if( recentFiles.count() < 5)
    {
      recentFiles.insert(0, file);
    }
    else
    {
      recentFiles.remove(4);
      recentFiles.insert(0, file);
    }
    recentFilesMenu->clear();
    for ( int i=0 ; i < (int) recentFiles.count(); i++)
    {
      recentFilesMenu->insertItem(recentFiles.at(i));
    }
  }
}

void KAuditorApp::openDocumentFile(const char* _cmdl)
{
  slotStatusMsg(i18n("Opening file..."));

  doc->openDocument(_cmdl);
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}


KAuditorDoc *KAuditorApp::getDocument() const
{
  return doc;
}

void KAuditorApp::saveOptions ()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", toolBar()->isVisible());
  config->writeEntry("Show Statusbar",statusBar()->isVisible());
  config->writeEntry("MenuBarPos", (int) menuBar()->menuBarPos());
  config->writeEntry("ToolBarPos", (int) toolBar()->barPos());
  config->writeEntry("Recent Files", recentFiles);

  config->setGroup("Training Options");
  config->writeEntry("trainCycles", mpParams->trainCycles);
}

void KAuditorApp::readOptions() {
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewMenu->setItemChecked(ID_VIEW_TOOLBAR, bViewToolbar);
  if(!bViewToolbar)
     enableToolBar(KToolBar::Hide);
	
  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewMenu->setItemChecked(ID_VIEW_STATUSBAR, bViewStatusbar);
  if(!bViewStatusbar)
    enableStatusBar(KStatusBar::Hide);

  // bar position settings
  KMenuBar::menuPosition menuBarPos;
  menuBarPos=(KMenuBar::menuPosition) config->readNumEntry("MenuBarPos", KMenuBar::Top);

  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);

  menuBar()->setMenuBarPos(menuBarPos);
  toolBar()->setBarPos(toolBarPos);
	
  // initialize the recent file list
  recentFiles.setAutoDelete(TRUE);
  config->readListEntry("Recent Files", recentFiles);
	
  for (int i=0; i < (int) recentFiles.count(); i++)
    recentFilesMenu->insertItem(recentFiles.at(i));

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
    resize(size);

	///////////////////////////////////////////////////////////////////////////////
	// KAuditor spesific options

	config->setGroup("Training Options");
	mpParams->trainCycles = config->readNumEntry("trainCycles");
}

void KAuditorApp::saveProperties(KConfig *_cfg)
{
  if(doc->getTitle()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    QString filename=doc->getAbsFilePath();	
    _cfg->writeEntry("filename", filename);
    _cfg->writeEntry("modified", doc->isModified());
		
    QString tempname = kapp->tempSaveName(filename);
    doc->saveDocument(tempname);
  }
}


void KAuditorApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
  	
    if(canRecover)
    {
      doc->openDocument(tempname);
      doc->setModified();
      QFileInfo info(filename);
      doc->setAbsFilePath(info.absFilePath());
      doc->setTitle(info.fileName());
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(filename);
    }
  }

  QString caption=kapp->getCaption();	
  setCaption(caption+": "+doc->getTitle());
}		

bool KAuditorApp::queryClose()
{
  return doc->saveModified();
}

bool KAuditorApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KAuditorApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));
	
  KAuditorApp *new_window= new KAuditorApp();
  new_window->show();

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KAuditorApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  if(!doc->saveModified()) {
     // here saving wasn't successful

  } else {	
    doc->newDocument();		

    QString caption=kapp->getCaption();	
    setCaption(caption+": "+doc->getTitle());
  }

	updateMenus ();
}

void KAuditorApp::slotFileOpen() {
	slotStatusMsg(i18n("Opening file..."));
	
	if(!doc->saveModified()) {
		// here saving wasn't successful
	} else {	
    	QString fileToOpen=KFileDialog::getOpenFileName(QDir::currentDirPath(),
	                                                    i18n("*|All files"), this, i18n("Open File..."));
		if(!fileToOpen.isEmpty()) {
			if (doc->openDocument(fileToOpen)) {
				// Open succeeded
				QString caption=kapp->getCaption();	
				setCaption(caption+": "+doc->getTitle());
			} else {
				// Open failed
			}
			addRecentFile(fileToOpen);
		}
	}

	updateMenus ();
}

void KAuditorApp::slotFileOpenRecent(int id_) {
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified()) {
     // here saving wasn't successful
  } else {
    doc->openDocument(recentFiles.at(id_));
    QString caption=kapp->getCaption();	
    setCaption(caption+": "+doc->getTitle());
  }

	updateMenus ();
}

void KAuditorApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));
	
  doc->saveDocument(doc->getAbsFilePath());

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KAuditorApp::slotFileSaveAs() {
  slotStatusMsg(i18n("Saving file with a new filename..."));

  QString newName=KFileDialog::getSaveFileName(QDir::currentDirPath(),
                                               i18n("*|All files"), this, i18n("Save as..."));
  if(!newName.isEmpty())
  {
    QFileInfo saveAsInfo(newName);
    doc->setTitle(saveAsInfo.fileName());
    doc->setAbsFilePath(saveAsInfo.absFilePath());
    doc->saveDocument(newName);
    addRecentFile(newName);

    QString caption=kapp->getCaption();	
    setCaption(caption+": "+doc->getTitle());
  }

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KAuditorApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KAuditorApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  QPrinter printer;
  if (printer.setup(this))
  {
    view->print(&printer);
  }

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KAuditorApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KTMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
	break;
    }
  }	
	updateMenus ();
}

void KAuditorApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));

	updateMenus ();
}

void KAuditorApp::slotEditCopy() {
	slotStatusMsg(i18n("Copying selection to clipboard..."));
	
	String buf;
	buf.reserve (10000);
	
	// Find out what the selected item is
	String selected;
	if (mSelected==i18n("Neural predictions"))
		selected = "neural";
	else if (mSelected==i18n("Avg delta predictions"))
		selected = "avgdp";

	// If a good item was selected
	if (selected!="") {
		// Copy the prediction results to clipboard buffer
		const Matrix& mtx = getDocument()->mPredictions[mSelected];
		for (int r=0; r<mtx.rows; r++) {
			for (int c=0; c<mtx.cols; c++) {
				if (c>0)
					buf += '\t';
				buf += String(mtx.get(r,c));
			}
			buf += '\n';
		}
	}

	QClipboard *cb = QApplication::clipboard();
	cb->setText( (CONSTR) buf);

	updateMenus ();
}

void KAuditorApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KAuditorApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggle the toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if( viewMenu->isItemChecked(ID_VIEW_TOOLBAR))
  {
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, false);
    enableToolBar(KToolBar::Hide);
  }
  else
  {
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, true);
    enableToolBar(KToolBar::Show);
  }		

	updateMenus ();
}

void KAuditorApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if( viewMenu->isItemChecked(ID_VIEW_STATUSBAR))
  {
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, false);
    enableStatusBar(KStatusBar::Hide);
  }
  else
  {
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, true);
    enableStatusBar(KStatusBar::Show);
  }
	
	updateMenus ();
}


void KAuditorApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}


void KAuditorApp::slotStatusHelpMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}



void KAuditorApp::commandCallback(int id_)
{
  switch (id_)
  {
    case ID_FILE_NEW_WINDOW:
         slotFileNewWindow();
    	 break;

    case ID_FILE_NEW:
    	 slotFileNew();
         break;

    case ID_FILE_OPEN:
         slotFileOpen();
         break;

    case ID_FILE_OPEN_BUDGET:
         slotFileOpenBudget();
         break;

    case ID_FILE_SAVE:
         slotFileSave();
         break;

    case ID_FILE_SAVE_AS:
         slotFileSaveAs();
         break;

    case ID_FILE_CLOSE:
         slotFileClose();
         break;

    case ID_FILE_OPEN_NETWORK:
         slotFileLoadNetwork();
         break;

    case ID_FILE_SAVE_NETWORK:
         slotFileSaveNetwork();
         break;

    case ID_FILE_PRINT:
         slotFilePrint();
         break;

    case ID_FILE_QUIT:
         slotFileQuit();
         break;

    case ID_EDIT_CUT:
         slotEditCut();
         break;

    case ID_EDIT_COPY:
         slotEditCopy();
         break;

    case ID_EDIT_PASTE:
         slotEditPaste();
         break;
  
    case ID_VIEW_TOOLBAR:
         slotViewToolBar();
         break;

    case ID_VIEW_STATUSBAR:
         slotViewStatusBar();
         break;

    case ID_OPTIONS_TRAIN_SETTINGS:
         slotDataTrainSettings();
         break;

    case ID_OPTIONS_SAVE:
         saveOptions();
         break;

    case ID_DATA_TRAIN:
         slotDataTrain();
         break;

    case ID_DATA_PLOT:
         slotDataPlot();
         break;

    case ID_DATA_STATS:
         slotDataStats();
         break;

    default:
         break;
  }
}

void KAuditorApp::statusCallback(int id_)
{
  switch (id_)
  {
    case ID_FILE_NEW_WINDOW:
         slotStatusHelpMsg(i18n("Opens a new application window"));
         break;

    case ID_FILE_NEW:
         slotStatusHelpMsg(i18n("Creates a new document"));
         break;

    case ID_FILE_OPEN:
         slotStatusHelpMsg(i18n("Opens an existing document"));
         break;

    case ID_FILE_OPEN_RECENT:
         slotStatusHelpMsg(i18n("Opens a recently used file"));
         break;

    case ID_FILE_OPEN_BUDGET:
         slotStatusHelpMsg(i18n("Opens a budget file"));
         break;

    case ID_FILE_SAVE:
         slotStatusHelpMsg(i18n("Saves the actual document"));
         break;

    case ID_FILE_SAVE_AS:
         slotStatusHelpMsg(i18n("Saves the actual document as..."));
         break;

    case ID_FILE_CLOSE:
         slotStatusHelpMsg(i18n("Closes the actual document"));
         break;

    case ID_FILE_PRINT:
         slotStatusHelpMsg(i18n("Prints out the actual document"));
         break;

    case ID_FILE_OPEN_NETWORK:
         slotStatusHelpMsg(i18n("Loads a saved trained neural network predictor"));
         break;

    case ID_FILE_SAVE_NETWORK:
         slotStatusHelpMsg(i18n("Saves a trained neural network predictor"));
         break;

    case ID_FILE_QUIT:
         slotStatusHelpMsg(i18n("Quits the application"));
         break;

    case ID_EDIT_CUT:
         slotStatusHelpMsg(i18n("Cuts the selected section and puts it to the clipboard"));
         break;

    case ID_EDIT_COPY:
         slotStatusHelpMsg(i18n("Copies the selected section to the clipboard"));
         break;

    case ID_EDIT_PASTE:
         slotStatusHelpMsg(i18n("Pastes the clipboard contents to actual position"));
         break;

    case ID_VIEW_TOOLBAR:
         slotStatusHelpMsg(i18n("Enables/disables the toolbar"));
         break;

    case ID_VIEW_STATUSBAR:
         slotStatusHelpMsg(i18n("Enables/disables the statusbar"));
         break;

    case ID_OPTIONS_TRAIN_SETTINGS:
         slotStatusHelpMsg(i18n("Neural network training settings"));
         break;

    case ID_OPTIONS_SAVE:
         slotStatusHelpMsg(i18n("Save application settings to configuration file now"));
         break;

    case ID_DATA_TRAIN:
         slotStatusHelpMsg(i18n("Trains the neural network predictor"));
         break;

    case ID_DATA_PLOT:
         slotStatusHelpMsg(i18n("Plot datasets"));
         break;

    case ID_DATA_STATS:
         slotStatusHelpMsg(i18n("Inspect prediction statistics"));
         break;

    default:
         break;
  }
}

//////////////////////////////////////////////////////////////////////////////

void KAuditorApp::slotFileOpenBudget () {
	slotStatusMsg(i18n("Opening budget..."));
	
	QString fileToOpen=KFileDialog::getOpenFileName(QDir::currentDirPath(),
                                                    i18n("*|All files"), this, i18n("Open Budget File..."));
	if(!fileToOpen.isEmpty())
		doc->openBudget(fileToOpen);

	updateMenus ();
}

void KAuditorApp::slotDataTrainSettings () {
	TrainerDlg* trainerDlg = new TrainerDlg (this, i18n("Neural Network Settings"), mpParams);
	trainerDlg->show ();
	delete trainerDlg;
}

void KAuditorApp::slotDataTrain () {
	doc->clearNeuralStats ();
	TrainProgrDlg* progr = new TrainProgrDlg (this, i18n("Training progress"));
	if (progr->exec ()) {
		// Accept
		mDefaultStatus = i18n("Inspect prediction results in Data menu.");
	} else {
		// Reject
		doc->clearNeuralStats ();
	}
	doc->computeNeuralStats ();
	delete progr;

	updateMenus ();
}

void KAuditorApp::slotDataPlot () {
	PlotDlg* plotDlg = new PlotDlg (this, i18n("Data plots"));
	plotDlg->show ();
	delete plotDlg;
}

void KAuditorApp::selChanged (QListViewItem* item) {
	mSelected = String(item->text(0));
	if (mSelected==i18n("Neural predictions") || mSelected==i18n("Avg delta predictions"))
		enableCommand(ID_EDIT_COPY);
	else
		disableCommand(ID_EDIT_COPY);
}

void KAuditorApp::slotDataStats () {
	StatsDlg* statsDlg = new StatsDlg (this, doc);
	statsDlg->show ();
	delete statsDlg;
}

void KAuditorApp::updateMenus () {
	if (doc->mLoaded.patterns>0) {
		enableCommand(ID_FILE_OPEN_BUDGET);
		enableCommand(ID_FILE_OPEN_RECENT_BUDGET);
		enableCommand(ID_FILE_OPEN_NETWORK);
		enableCommand(ID_OPTIONS_TRAIN_SETTINGS);
		enableCommand(ID_DATA_TRAIN);
		enableCommand(ID_DATA_STATS);
		enableCommand(ID_DATA_PLOT);
		mDefaultStatus = i18n("Train neural network from Data/Train.");
	} else {
		disableCommand(ID_FILE_OPEN_BUDGET);
		disableCommand(ID_FILE_OPEN_RECENT_BUDGET);
		disableCommand(ID_FILE_OPEN_NETWORK);
		disableCommand(ID_OPTIONS_TRAIN_SETTINGS);
		disableCommand(ID_DATA_TRAIN);
		disableCommand(ID_DATA_STATS);
		disableCommand(ID_DATA_PLOT);
		mDefaultStatus = i18n("Open balance file from File/Open or File/Open recent.");
	}
	if (doc->mpNeuralStrategy) {
		enableCommand(ID_FILE_SAVE_NETWORK);
	} else {
		disableCommand(ID_FILE_SAVE_NETWORK);
	}
	
	slotStatusMsg ((CONSTR) mDefaultStatus);
}

void KAuditorApp::slotFileLoadNetwork() {
	slotStatusMsg(i18n("Opening network file..."));
	
	QString fileToOpen=KFileDialog::getOpenFileName(QDir::currentDirPath(),
	                                                i18n("*.net|All files"), this, i18n("Open File..."));
	if(!fileToOpen.isEmpty()) {
		if (doc->loadNetwork(fileToOpen)) {
			// Open succeeded
		} else {
			// Open failed
		}
	}
	updateMenus ();
}

void KAuditorApp::slotFileSaveNetwork() {
	slotStatusMsg(i18n("Saving network with a new filename..."));

	QString newName=KFileDialog::getSaveFileName(QDir::currentDirPath(),
                                               i18n("*.net|SNNS network files"), this, i18n("Save as..."));
	if(!newName.isEmpty()) {
		QFileInfo saveAsInfo(newName);
		doc->setTitle(saveAsInfo.fileName());
		doc->setAbsFilePath(saveAsInfo.absFilePath());
		doc->saveNetwork (newName);
		mDefaultStatus = i18n("Inspect prediction results in Data menu.");
	}
	updateMenus ();
}
