/***************************************************************************
                          kauditordoc.cpp  -  description
                             -------------------
    begin                : Tue Jul 18 12:38:24 EEST 2000
    copyright            : (C) 2000 by Marko Grnroos
    email                : magi@iki.fi
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define format strformat
#include <magic/object.h>
#include <patternset.h>
#include <equalization.h>
#include <annfilef.h>
#include <magic/cstring.h>
#include <auditing.h>
#include <magic/pararr.h>
#undef ASSERT
#undef format

#include <ctype.h>
#include <fstream.h>

// include files for Qt
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>

// include files for KDE
#include <kapp.h>
#include <kmsgbox.h>

// application specific includes
#include "kauditordoc.h"
#include "kauditor.h"
#include "kauditorview.h"

QList<KAuditorView> *KAuditorDoc::pViewList = 0L;

KAuditorDoc::KAuditorDoc(QWidget *parent, const char *name) : QObject(parent, name) {
	if(!pViewList)
		pViewList = new QList<KAuditorView>();

	mpNeuralStats = NULL;
	mpNeuralStrategy = NULL;
}

KAuditorDoc::~KAuditorDoc() {
	delete mpNeuralStrategy;
	clearResults ();
}

void KAuditorDoc::addView(KAuditorView *view) {
  pViewList->append(view);
}

void KAuditorDoc::removeView(KAuditorView *view) {
  pViewList->remove(view);
}

void KAuditorDoc::setAbsFilePath(const QString &filename) {
  absFilePath=filename;
}

const QString &KAuditorDoc::getAbsFilePath() const {
  return absFilePath;
}

void KAuditorDoc::setTitle(const QString &_t) {
  title=_t;
}

const QString &KAuditorDoc::getTitle() const {
  return title;
}

void KAuditorDoc::slotUpdateAllViews(KAuditorView *sender) {
  KAuditorView *w;
  if(pViewList)  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())    {
      if(w!=sender)
        w->repaint();
    }
  }

}

bool KAuditorDoc::saveModified() {
  bool completed=true;

  if(modified)   {
    KAuditorApp *win=(KAuditorApp *) parent();
    int want_save = KMsgBox::yesNoCancel(win, i18n("Warning"),
                                         i18n("The current file has been modified.\n"
                                              "Do you want to save it?"));
    switch(want_save) {
      case 1:
           if (title == i18n("Untitled")) {
             win->slotFileSaveAs();
           } else {
             saveDocument(getAbsFilePath());
       	   };

       	   deleteContents();
           completed=true;
           break;

      case 2:
           setModified(false);
           deleteContents();
           completed=true;
           break;	

      case 3:
           completed=false;
           break;

      default:
           completed=false;
           break;
    }
  }

  return completed;
}

void KAuditorDoc::closeDocument()
{
  deleteContents();
}

bool KAuditorDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  absFilePath=QDir::homeDirPath();
  title=i18n("Untitled");

  return true;
}

bool KAuditorDoc::openDocument(const QString &filename, const char *format /*=0*/) {
	QFileInfo fileInfo(filename);
	title=fileInfo.fileName();
	absFilePath=fileInfo.absFilePath();
	
	// Open the file.
	ifstream in ((CONSTR) filename);

	/////////////////////////////////////////////////////////////////////
	// Read variable names from the first row of the file

	// Read names from file	
	String namestr;
	getS (in, namestr);
	Array<String> names;
	namestr.split (names, '\t');
	
	// Check for consistency
    KAuditorApp *win=(KAuditorApp *)parent();
	if (names.size<2) {
		KMsgBox::message (win, i18n("Error"), i18n("Error in monthly balance file format:\nToo few tab-separated columns in the first row"));
		return false;
	} else if (names[0].len>0) {
		KMsgBox::message (win, i18n("Error"), i18n("Error in monthly balance file format:\nThe first field in first row is not empty. The file has probably wrong format."));
		return false;
	}
	
	// Convert field names to lowercase (first letter uppercase)
	for (int i=1; i<names.size; i++) {
		// Check for consistency
		if (names[i].len==0) {
			KMsgBox::message (win, i18n("Error"), strformat(i18n("Error in monthly balance file format:\n"
					"Empty account name found (column %d).\nThis is probably an error."), i+1));
			return false;
		}
		
		// Convert
		for (int j=0; j<names[i].len; j++)
			if (j==0)
				names[i][j] = toupper (names[i][j]);
			else
				names[i][j] = tolower (names[i][j]);
		printf ("%s\n", (CONSTR) names[i]);
	}
	names.removeFill (0); // The first column should be empty
	
	mVariableNames = names;

	// Put column names in the view
	pViewList->first()->makeColumns();

	//////////////////////////////////////////////////////	
	// Load data from the file	
	mLoaded.make (0,mVariableNames.size+1,0);
	mLoaded.load (in, (CONSTR)absFilePath);
	in.close ();

	//////////////////////////////////////////////////////	
	// Check for consistency

	// Check that there are enough months	
	if (mLoaded.patterns<36) {
		KMsgBox::message (win, i18n("Error"), (CONSTR) strformat (i18n("Error in monthly balance file:\n"
					"File has %d months of data, 36 months is the absolute minimum.\n"), mLoaded.patterns));
		return false;
	}
	
	// Check that the first column is
	//     a) numbers between 0001-9912
	//     b) sequentially ordered
	//     c) final year has all 12 months
	int prevdate = int (mLoaded.input(0,0)+0.001);
	for (int i=0; i<mLoaded.patterns; i++) {
		int date = int (mLoaded.input(i,0)+0.001);
		
		// Check bounds
		if ((date/100)<0 || (date/100)>99 || (date%100)<1 || (date%100)>12) {
			KMsgBox::message (win, i18n("Error"), (CONSTR) strformat (i18n("Error in monthly balance file:\n"
						"Record %d date %04d is out of bounds. Format is YYMM, where MM is 01-12.\n"), i+1, date));
			return false;
		}
		
		// Check sequentality
		if (i>0 &&
				!((date%100==(prevdate%100)+1 && date/100==prevdate/100) || // Months are sequental OR
				  ((date%100==1 && prevdate%100==12 && (date/100)%100==((prevdate/100)+1)%100)))) { // Year changed
			KMsgBox::message (win, i18n("Error"), (CONSTR) strformat (i18n("Error in monthly balance file:\n"
						"Record %d date %04d is inconsistent with previous month %04d.\n"), i+1, date, prevdate));
			return false;
		}
		
		// Check that the final year has 12 months
		if (i==mLoaded.patterns-1 && date%100!=12) {
			KMsgBox::message (win, i18n("Error"), (CONSTR) strformat (i18n("Error in monthly balance file:\n"
						"Final year %02d didn't have 12 months (last month was %02d)\n"), date/100, date%100));
			return false;
		}
		
		prevdate = date;
	}
	
	
	//////////////////////////////////////////////////////
	// Finish
	
	((KAuditorApp *) parent())->statusBar()->message (QString((CONSTR)strformat(i18n("Loaded %d samples of length %d\n"), mLoaded.patterns, mLoaded.inputs+mLoaded.outputs)),10000);
	fflush(stdout);

	// Update view	
	dataChanged ();
	slotUpdateAllViews (NULL);
	
	modified=false;
	return true;
}

bool KAuditorDoc::saveDocument(const QString &filename, const char *format /*=0*/) {
  /////////////////////////////////////////////////
  // TODO: Add your document saving code here
  /////////////////////////////////////////////////

  modified=false;
  return true;
}

void KAuditorDoc::deleteContents() {
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}

bool KAuditorDoc::openBudget (const QString &filename, const char *format=0) {
	// Open the file.
	ifstream in ((CONSTR) filename);

	// Skip the first row
	String namestr;
	getS (in, namestr);

	// Read the file as a pattern set (we don't have a method to read matrices)
	PatternSet budgetSet;
	budgetSet.make (0,mVariableNames.size+1,0);
	budgetSet.load (in, (CONSTR)absFilePath);
	
	if (budgetSet.patterns>12)
		KMsgBox::message (NULL, i18n("Warning"), i18n("Warning: budget file has too many values. Using 12 first."));
	
	// Convert it to a matrix and place among other datasets
	mPredictions.set ("budget", new Matrix (budgetSet.getMatrix ().sub(0,11,1,Matrix::end)));
	
	slotUpdateAllViews (NULL);
	return true;
}

void KAuditorDoc::dataChanged () {
	clearResults ();
	trainTestStrategy ("avgdp", NULL);
	trainTestStrategy ("zerodp", NULL);
	trainTestStrategy ("previous", NULL);
	trainTestStrategy ("avg", NULL);
}

void KAuditorDoc::trainTestStrategy (const String& strategyName, TrainingObserver* observer) {
    KAuditorApp *win=(KAuditorApp *) parent();

	//////////////////////////////////////////////////////////////////////////////////
	// Create strategy and give it all necessary parameters

	AuditingStrategy* strategy = strategyFactory (strategyName, *win->mpParams);
	
	// For ANN training, we want to observe the training in TrainProgrDlg
	if (AbsoluteNeuralPrediction* anp = dynamic_cast<AbsoluteNeuralPrediction*>(strategy)) {
		anp->setObserver (observer);
		
		// Store the strategy
		delete mpNeuralStrategy;
		mpNeuralStrategy = anp;
	}

	//////////////////////////////////////////////////////////////////////////////////
	// Train the prediction strategy
	
	// Prepare training data
	Matrix traindata = mLoaded.getMatrix().sub(0,mLoaded.getMatrix().rows-13, 1, Matrix::end);
	
	printf ("training\n");
	strategy->train (traindata, mLoaded.getMatrix().get(0,0));
	printf ("trained\n");
		
	//////////////////////////////////////////////////////////////////////////////////
	// Make predictions with the strategy
	
	// Test and store results
	testStrategy (strategy, strategyName, mLoaded.getMatrix());
}

AuditingStrategy* KAuditorDoc::strategyFactory (const String& strategyName, const TrainParameters& rParams) const {
	// Get instance of the strategy library
	AuditingStrategyLib& audStr = AuditingStrategyLib::instance();
		
	// Create strategy	
	int strategyId;
	if (strategyName == "avgdp")
		strategyId = 0;
	else if (strategyName == "zerodp")
		strategyId = 1;
	else if (strategyName == "neural") {
		if (rParams.allOutputs==true)
			strategyId = 2;
		else
			strategyId = 3;
	} else if (strategyName == "previous")
		strategyId = 4;
	else if (strategyName == "avg")
		strategyId = 5;
	else
		throw Exception (strformat ("Strategy %s not exists", (CONSTR) strategyName));

	fprintf (stderr, "Creating strategy %s/%d\n", (CONSTR) strategyName, strategyId);
	
	AuditingStrategy* strategy = audStr.create(strategyId);
	initStrategy (strategy, rParams);
	return strategy;
}

void KAuditorDoc::initStrategy (AuditingStrategy* strategy, const TrainParameters& rParams) const {
	// Give parameters to the prediction strategy
	StringMap params;
	params.set ("maxCycles", rParams.trainCycles);
	params.set ("validationInterval", "10");
	params.set ("terminator", "-UP5");
	params.set ("inputMonths", String(int(rParams.inputMonths)));
	params.set ("AbsoluteNeuralPrediction.useAllOutputs", String(int(rParams.allOutputs)));
	params.set ("AbsoluteNeuralPrediction.useAllInputs", String(int(rParams.allInputs)));
	params.set ("AbsoluteNeuralPrediction.globalEqualization", String(int(rParams.equalizeGlobal)));
	params.set ("AbsoluteNeuralPrediction.hidden", rParams.hiddenString());
	params.set ("RPropTrainer.delta0", String(rParams.delta0));
	params.set ("RPropTrainer.deltamax", String(rParams.deltaMax));
	params.set ("BackpropTrainer.decay", String(rParams.useWeightDecay? rParams.weightDecay:1.0));
	params.set ("BackpropTrainer.batchLearning", "1");
	strategy->make (params);
}

void KAuditorDoc::testStrategy (AuditingStrategy* strategy, const String& strategyName, const Matrix& alldata) {
	ASSERTWITH (strategy, "No prediction strategy passed.");

	// Prepare test data
	Matrix testdata = alldata.sub(alldata.rows-12-strategy->inputMonths(), Matrix::end, 1, Matrix::end);
	int testmonth = int (alldata.get(alldata.rows-12-strategy->inputMonths(),0));

	fprintf (stderr, "Making predictions %s...\n", (CONSTR) strategyName);
	
	// Make predictions		
	AuditingTestResults* results = strategy->test (testdata, testmonth);
	fprintf (stderr, "Tested %s, reporting...\n", (CONSTR) strategyName);
	
	// Store the prediction results according to their type
	if (strategyName == "neural") {
		// Neural network methods
		if (!mpNeuralStats)
			mpNeuralStats = new Array<AuditingTestResults> ();
		// Add one run worth of results.
		// The results array should have been cleared before training.
		mpNeuralStats->add (results);
	} else
		// Zero or Average delta prediction methods
		mStats.set (strategyName, results);
		
	// Print debugging data
	printf ("Average error: %f\n", results->averageError);
	for (int v=0; v<results->averageErrors.size; v++)
		printf ("Variable %d: %f\n", v, results->averageErrors[v]);

	// Make predictions
	printf ("Predicting %s...\n", (CONSTR) strategyName);
	Matrix* result = new Matrix (strategy->predict (testdata, testmonth));
	sout << *result;
	mPredictions.set (strategyName, result);
	if (strategyName == "neural")
		slotUpdateAllViews (NULL);
	
	printf ("Predicted %s.\n", (CONSTR) strategyName);
}

void KAuditorDoc::clearResults () {
	fprintf (stderr, "Clearing all results!\n");
	clearNeuralStats ();
	mPredictions.remove ("avgdp");
	mPredictions.remove ("zerodp");
	mPredictions.remove ("previous");
	mPredictions.remove ("avg");
	mPredictions.remove ("budget");
}

void KAuditorDoc::clearNeuralStats () {
	mStats.empty ();
	mPredictions.remove ("neural");
	delete mpNeuralStrategy;
	mpNeuralStrategy = NULL;
}

void addResultNames (Array<Array<String> >* resultarr, CONSTR name, CONSTR desc) {
	Array<String>* newr = new Array<String> ();
	newr->add (new String (name));
	newr->add (new String (desc));
	resultarr->add (newr);
}

Array<Array<String> >* KAuditorDoc::listResultNames () const {
	Array<Array<String> >* result = new Array<Array<String> > ();
	
	addResultNames (result, "zerodp", i18n("Zero delta prediction"));
	addResultNames (result, "avgdp", i18n("Average delta prediction"));
	if (mPredictions.hasKey("previous"))
		addResultNames (result, "previous", i18n("Previous year's values"));
	if (mPredictions.hasKey("avg"))
		addResultNames (result, "avg", i18n("Average of previous years"));
	if (mPredictions.hasKey("budget"))
		addResultNames (result, "budget", i18n("Budgeted values"));
	if (mPredictions.hasKey("neural"))
		addResultNames (result, "neural", i18n("Neural network prediction"));
	
	return result;
}

void KAuditorDoc::computeNeuralStats () {
	const Array<AuditingTestResults>& allResults = *mpNeuralStats;

	// Create intermediate data containers
	Vector averageErrors (allResults.size);								// For all variables
	Array<Vector> averageVarErrors (allResults[0].averageErrors.size);	// For each variable
	for (int i=0; i<averageVarErrors.size; i++)			// Create the vectors
		averageVarErrors[i].make (allResults.size);
	
	// Create stats object
	AuditingTestResults* stats = new AuditingTestResults ();
	stats->make (averageVarErrors.size);
	
	// Combine multiple-run stats into single stats structure
	for (int i=0; i<allResults.size; i++) {				// Put the averages into the vectors
		averageErrors[i] = allResults[i].averageError;
		for (int j=0; j<averageVarErrors.size; j++) {
			averageVarErrors[j][i] = allResults[i].averageErrors[j];
			stats->addValue (j, allResults[i].averageErrors[j]);
		}
	}

	// Place the stats among others
	mStats.set ("neural", stats);
}

bool KAuditorDoc::loadNetwork (const QString &filename, const char *format=0) {
	clearNeuralStats ();

	ANNetwork* network = new ANNetwork;
	try {
		// Load the network
		ANNFileFormatLib::load ((CONSTR) filename, *network);
	} catch (...) {
		delete network;
		KMsgBox::message ((KAuditorApp*)parent(), i18n("Error"), i18n("File open failed for some reason."));
		return false;
	}

	// Create strategy
	if (AuditingStrategy* strategy = strategyFactory ("neural", *((KAuditorApp*) parent())->mpParams)) {
		if ((mpNeuralStrategy = dynamic_cast<AbsoluteNeuralPrediction*> (strategy))) {
			// Give the loaded network to the strategy
			mpNeuralStrategy->setNetwork (network);
		
			// Test and store results
			testStrategy (mpNeuralStrategy, "neural", mLoaded.getMatrix());
		
			computeNeuralStats ();
		
			KMsgBox::message ((KAuditorApp*)parent(), i18n("Success"), i18n("Prediction done with the saved neural predictor."));
		} else {
			// Strategy object was wrong class for some reason
			delete network;
			delete strategy;
			KMsgBox::message ((KAuditorApp*)parent(), i18n("Error"), i18n("Predictor couldn't be created for some reason."));
			return false;
		}
	} else {
		// Strategy couldn't be created, for some reason
		delete network;
		KMsgBox::message ((KAuditorApp*)parent(), i18n("Error"), i18n("Predictor couldn't be created for some reason."));
		return false;
	}

	return true;
}

bool KAuditorDoc::saveNetwork			(const QString &filename, const char *format=0) const {
	try {
		if (mpNeuralStrategy) {
			ANNFileFormatLib::save ((CONSTR) filename, mpNeuralStrategy->getNetwork(), "SNNS");
		} else
			return false;
	} catch (...) {
		return false;
	}
	return true;
}
