/***************************************************************************
                          kauditordoc.h  -  description
                             -------------------
    begin                : Tue Jul 18 12:38:24 EEST 2000
    copyright            : (C) 2000 by Marko Grnroos
    email                : magi@iki.fi
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KAUDITORDOC_H
#define KAUDITORDOC_H

#define format strformat
#include <magic/object.h>
#include <magic/map.h>
#include <patternset.h>
#include <auditing.h>
#undef format
#undef ASSERT

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>

// forward declaration of the KAuditor classes
class KAuditorView;
class TrainingObserver; // In trainer.h (inanna)
class TrainParameters;	// In kauditor.h

/**	KAuditorDoc provides a document object for a document-view model.
  *
  * The KAuditorDoc class provides a document object that can be used in conjunction with the classes KAuditorApp and KAuditorView
  * to create a document-view model for standard KDE applications based on KApplication and KTMainWindow. Thereby, the document object
  * is created by the KAuditorApp instance and contains the document structure with the according methods for manipulation of the document
  * data by KAuditorView objects. Also, KAuditorDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 0.4 code generation
  */
class KAuditorDoc : public QObject {
  Q_OBJECT
  public:
    /** Constructor for the fileclass of the application */
    KAuditorDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~KAuditorDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(KAuditorView *view);
    /** removes a view from the list of currently connected views */
    void removeView(KAuditorView *view);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true){ modified=_m; };
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** "save modified" - asks the user for saving if the document is modified */
    bool saveModified();	
    /** deletes the document's contents */
    void deleteContents();
    /** initializes the document generally */
    bool newDocument();
    /** closes the acutal document */
    void closeDocument();
    /** loads the document by filename and format and emits the updateViews() signal */
    bool openDocument(const QString &filename, const char *format=0);
    /** saves the document under filename and format.*/	
    bool saveDocument(const QString &filename, const char *format=0);
    /** sets the path to the file connected with the document */
    void setAbsFilePath(const QString &filename);
    /** returns the pathname of the current document file*/
    const QString &getAbsFilePath() const;
    /** sets the filename of the document */
    void setTitle(const QString &_t);
    /** returns the title of the document */
    const QString &getTitle() const;

  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void slotUpdateAllViews(KAuditorView *sender);
 	
  public:	
    /** the list of the views currently connected to the document */
    static QList<KAuditorView> *pViewList;	

  private:
    /** the modified flag of the current document */
    bool	modified;
    QString	title;
    QString	absFilePath;

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Application-specific parameters

  public:

    /** Does all operations which are needed when the data changes. */
	void					dataChanged			();
	
    /** Trains and tests given strategy. */
	void					trainTestStrategy	(const String& strategyName, TrainingObserver* observer);
	
	/** Returns a list of available test results. */
	Array<Array<String> >*	listResultNames		() const;
	
	/** Clears all test results. */
	void					clearResults		();
	
	/** Clears neural result statistics. */
    void					clearNeuralStats	();

    /** Computes neural result statistics. */
    void					computeNeuralStats	();

	/** Loads given network from a file and tests the strategy with currently loaded data. */
	bool					loadNetwork			(const QString &filename, const char *format=0);
	
	/** Saves trained neural network to a file. */
	bool					saveNetwork			(const QString &filename, const char *format=0) const;

	/** Loads given budget data file. */	
    bool					openBudget			(const QString &filename, const char *format=0);

  private:

	/** Creates and initializes strategy by name and parameters. */
	AuditingStrategy*		strategyFactory		(const String& strategyName, const TrainParameters& rParams) const;

	/** Initializes given strategy with given parameters */	
	void					initStrategy		(AuditingStrategy* strategy, const TrainParameters& rParams) const;
	
	/** Tests strategy with given data and stores the results. */
	void					testStrategy		(AuditingStrategy* strategy, const String& strategyName, const Matrix& alldata);

  public:
  	Array<String>					mVariableNames;
	PatternSet						mLoaded;
	Map<String,Matrix>				mPredictions;
	Map<String,AuditingTestResults>	mStats;
	Array<AuditingTestResults>*		mpNeuralStats;
	AbsoluteNeuralPrediction*		mpNeuralStrategy;	// We store the neural strategy to be able to save it
};

#endif // KAUDITORDOC_H

