/***************************************************************************
                          kauditorview.cpp  -  description
                             -------------------
    begin                : Tue Jul 18 12:38:24 EEST 2000
    copyright            : (C) 2000 by Marko Grnroos
    email                : magi@iki.fi
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define format strformat
#include <magic/object.h>
#include <patternset.h>
#undef ASSERT
#undef format

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>

// application specific includes
#include "kauditorview.h"
#include "kauditordoc.h"
#include "kauditor.h"

KAuditorView::KAuditorView(QWidget *parent, const char *name) : QListView(parent, name)
{
	setBackgroundMode(PaletteBase);
	mColumns=0;
	connect (this, SIGNAL(selectionChanged(QListViewItem*)), (KAuditorApp*) parent, SLOT(selChanged(QListViewItem*)));
}

KAuditorView::~KAuditorView()
{
}

KAuditorDoc *KAuditorView::getDocument() const
{
  KAuditorApp *theApp=(KAuditorApp *) parentWidget();

  return theApp->getDocument();
}

void KAuditorView::print(QPrinter *pPrinter)
{
  QPainter printpainter;
  printpainter.begin(pPrinter);
	
  // TODO: add your printing code here

  printpainter.end();
}

void KAuditorView::repaint () {
    clear ();
	setSorting (-1);

    const PatternSet& loaded = getDocument()->mLoaded;
    QListViewItem* current=NULL;

    // Add dataset
    for (int i=0; i<2; i++) {
        current = new QListViewItem (this, current);
        if (i==0)
            current->setText (0, i18n("Inspected year"));
        else
            current->setText (0, i18n("Previous years"));

        // Add monthly data items
        QListViewItem* item=NULL;
        for (int r= (i? 0:(loaded.patterns-12)); r<(i?(loaded.patterns-12):loaded.patterns); r++) {
			item = new QListViewItem (current, item);
            item->setText (0, QString((CONSTR)strformat("%04d", int(loaded.input(r,0)+0.01))));
            for (int c=1; c<loaded.inputs; c++)
                item->setText (c, QString((CONSTR)strformat("%.2f", loaded.input(r,c))));
        }
    }

    // Add prediction results
    for (int i=0; i<3; i++) {
    	String name, fullname;
    	switch (i) {
    		case 0: name="budget"; fullname=i18n("Budget"); break;
    		case 1: name="neural"; fullname=i18n("Neural predictions"); break;
    		case 2: name="avgdp"; fullname=i18n("Avg delta predictions"); break;
    	};
	    if (const Matrix* results = getDocument()->mPredictions.getp(name)) {
		    QListViewItem* adp = new QListViewItem (this, current, (CONSTR) fullname);
		    QListViewItem* item = NULL;
		    for (int r=0; r<results->rows; r++) {
				item = new QListViewItem (adp, item);
				item->setText (0, (CONSTR)strformat("%04d", int(loaded.input(loaded.patterns-12+r,0)+0.01)));
				for (int c=0; c<results->cols; c++)
					item->setText (c+1, (CONSTR)strformat("%.2f", results->get(r,c)));
		    }
		}
	}

    QListView::repaint ();
}

void KAuditorView::makeColumns () {
	clear ();

	for (; mColumns < getDocument()->mVariableNames.size+1; mColumns++)
		addColumn (mColumns==0? i18n("Yr+Mo"):(CONSTR) getDocument()->mVariableNames[mColumns-1]);
	
	// Add new columns
	setColumnText (0, i18n("Yr+Mo"));
	for (int i=0; i<mColumns-1; i++)
		if (i<getDocument()->mVariableNames.size) {
			setColumnText (i+1, (CONSTR) getDocument()->mVariableNames[i]);
			setColumnWidthMode (i+1, QListView::Maximum);
		} else
			setColumnText (i+1, "");
}

