/***************************************************************************
                          statsdlg.cpp  -  description
                             -------------------
    begin                : Sun Jul 30 2000
    copyright            : (C) 2000 by Marko Grnroos
    email                : magi@iki.fi
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#define format strformat
#include <auditing.h>
#undef ASSERT
#undef format

#include <kapp.h>
#include "qclipboard.h"
#include "statsdlg.h"
#include "kauditordoc.h"

class StatsListViewItem : public QListViewItem {
  public:
		StatsListViewItem (QListView* parent, StatsListViewItem* previous, float limit, CONSTR column1)
			: QListViewItem (parent, previous, column1), mLimit (limit) {
		}

	virtual void	paintCell	(QPainter * p, const QColorGroup & cg, int column, int width, int align );
  private:
	float	mLimit;
};

void StatsListViewItem::paintCell (QPainter * p, const QColorGroup & cg, int column, int width, int align ) {
	float value = float(String(text (column)));
	bool emph = (column>0 && value>mLimit);
	QColorGroup localcg (cg.foreground(), cg.background(), cg.light(), cg.dark(), cg.mid(), emph? red:cg.text(), cg.base());
	p->setPen (blue);
	QListViewItem::paintCell (p, localcg, column, width, align);
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////

StatsDlg::StatsDlg(QWidget *parent, KAuditorDoc* doc) : QDialog(parent, i18n("Prediction statistics"),true){
	pDoc = doc;

	initDialog();
	
	QTab* qtab1 = new QTab ();
	qtab1->label = i18n("Errors per account");

	QTab* qtab2 = new QTab ();
	qtab2->label = i18n("Monthly errors");
	
	wViewTab = new QTabBar (this);
	wViewTab->setGeometry(10,55,380,25);
	wViewTab->setMinimumSize(0,0);
	wViewTab->addTab (qtab1);
	wViewTab->addTab (qtab2);

	wErrorList = new QListView(this,"NoName");
	wErrorList->setGeometry(10,80,780,290);
	wErrorList->setMinimumSize(0,0);
	wErrorList->setSorting (-1);
	wErrorList->addColumn (i18n("Month"));
	for (int v=0; v<pDoc->mVariableNames.size; v++)
		wErrorList->addColumn ((CONSTR) pDoc->mVariableNames[v]);
	wErrorList->hide();
	
	// Insert result set names	
	mpResultNames = pDoc->listResultNames ();
	for (int i=0; i<mpResultNames->size; i++)
		wResultList->insertItem ((CONSTR) (*mpResultNames)[i][1]);

	connect (wExit, SIGNAL(clicked()), this, SLOT(accept()));
	connect (wCopy, SIGNAL(clicked()), this, SLOT(copyResults()));
	connect (wResultList, SIGNAL(activated(int)), this, SLOT(listResults()));
	connect (wViewTab, SIGNAL(selected(int)), this, SLOT(tabSelect(int)));
	connect (wEmphasisLimit, SIGNAL(valueChanged(int)), this, SLOT(listResults()));
	
	wVarList->setSorting(-1);		
	listResults();
}

StatsDlg::~StatsDlg(){
	delete mpResultNames;
}

void StatsDlg::listResults() {
	// Determine name of the appropriate result matrix
	String strategyName = (*mpResultNames)[wResultList->currentItem()][0];
	
	// Fill text result widgets
	if (pDoc->mStats.hasKey(strategyName))
		wAvgErrorLabel->setText((CONSTR) strformat (i18n("Average error for all variables: %4.2f"), pDoc->mStats[strategyName].averageError));
	else
		wAvgErrorLabel->setText("");

	if (pDoc->mpNeuralStats)
		wRunsLabel->setText((CONSTR) strformat (i18n("Averaged from %d runs"), pDoc->mpNeuralStats->size));
	else
		wRunsLabel->setText("");

	// Per-variable errors
	wVarList->clear ();
	QListViewItem* previous = NULL;
	if (pDoc->mStats.hasKey(strategyName))
		for (int v=0; v<pDoc->mVariableNames.size; v++)
			previous = new QListViewItem (wVarList, previous, (CONSTR) pDoc->mVariableNames[v],
							   (CONSTR)String(pDoc->mStats[strategyName].averageErrors[v]), (CONSTR)String(0.0),
							   (CONSTR)String(0.0), (CONSTR)String(0.0));

	// Calculate per-variable sums
	Array<float> sums (pDoc->mVariableNames.size);
	for (int v=0; v<pDoc->mVariableNames.size; v++)
		sums[v] = 0.0;
	
	// Monthly errors for each variable
	wErrorList->clear ();
	StatsListViewItem* sprevious = NULL;
	float limit = float(String(wEmphasisLimit->text()));;
	for (int m=0; m<12; m++) {
		StatsListViewItem* item = new StatsListViewItem (wErrorList, sprevious, limit, (CONSTR) String(m+1));
		for (int v=0; v<pDoc->mVariableNames.size; v++) {
			float predicted = pDoc->mPredictions[strategyName].get(m,v);
			float actual = pDoc->mLoaded.input(pDoc->mLoaded.patterns-12+m, v+1);
			float error = int(fabs((predicted - actual)/actual)*100*10)/10.0;
			item->setText (v+1, (CONSTR) strformat("%3.2f%%", error));
			sprevious = item;
			sums[v] += error;
		}
	}
	
	// Per-variable averages
	StatsListViewItem* item = new StatsListViewItem (wErrorList, sprevious, limit, "AVERAGE");
	for (int v=0; v<pDoc->mVariableNames.size; v++)
			item->setText (v+1, (CONSTR) strformat("%3.2f%%", sums[v]/12));
}

void StatsDlg::copyResults () {
	String buf;
	buf.reserve (10000);
	
	if (false) {
		// Print per-variable results
	} else {
		// Print monthly errors for each variable
		for (QListViewItem* current = wErrorList->firstChild (); current; current = current->nextSibling ()) {
			buf += "Month ";
			for (int v=0; v<pDoc->mVariableNames.size; v++)
				buf += strformat ("\t%s", (CONSTR) current->text(v));
			buf += '\n';
		}
	}

	QClipboard *cb = QApplication::clipboard();
	cb->setText( (CONSTR) buf);
}

void StatsDlg::tabSelect (int i) {
	if (i==0) {
		wVarList->show ();
		wErrorList->hide ();
	} else {
		wVarList->hide ();
		wErrorList->show ();
	}
}
