/***************************************************************************
                          trainerdlg.cpp  -  description
                             -------------------
    begin                : Fri Jul 21 2000
    copyright            : (C) 2000 by Marko Grnroos
    email                : magi@iki.fi
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "trainerdlg.h"
#include "trainprogrdlg.h"
#include "kauditor.h"
#include "kauditordoc.h"

TrainerDlg::TrainerDlg(KAuditorApp *parent, const char *name, TrainParameters* params) : QDialog(parent,name,true){
	initDialog();

	pParams = params;
	mVariables = parent->getDocument()->mVariableNames.size;
	paramsToWidgets (pParams);
	
	connect (wExit, SIGNAL(clicked()), this, SLOT(accept()));
	connect (wCancel, SIGNAL(clicked()), this, SLOT(reject()));
	connect (wUseDefaults, SIGNAL(clicked()), this, SLOT(defaults()));
	connect (wUseWeightDecay, SIGNAL(clicked()), this, SLOT(useWeightDecayChanged()));
	connect (wInputsAll, SIGNAL(toggled(bool)), this, SLOT(inputsToggled()));
	connect (wOutputsAll, SIGNAL(toggled(bool)), this, SLOT(inputsToggled()));
	connect (wHidden1, SIGNAL(valueChanged(int)), this, SLOT(topologyChanged()));
	connect (wHidden2, SIGNAL(valueChanged(int)), this, SLOT(topologyChanged()));
	connect (wHidden3, SIGNAL(valueChanged(int)), this, SLOT(topologyChanged()));
	connect (wInputMonth, SIGNAL(toggled(bool)), this, SLOT(topologyChanged()));
	connect (wInputMonths, SIGNAL(valueChanged(int)), this, SLOT(topologyChanged()));
}

TrainerDlg::~TrainerDlg(){
}

void TrainerDlg::paramsToWidgets (const TrainParameters* params) {
	// Read current training parameters into widgets
	wDelta0			->setText		((CONSTR) String(params->delta0));
	wDeltaMax		->setText		((CONSTR) String(params->deltaMax));
	wUseWeightDecay	->setChecked	(params->useWeightDecay);
	wWeightDecay	->setText		((CONSTR) String(params->weightDecay));
	wTrainCycles	->setValue		(params->trainCycles);
	wRuns			->setValue		(params->runs);
	wInputMonth		->setChecked	(params->monthInputs);
	wInputsSingle	->setChecked	(!params->allInputs);
	wInputsAll		->setChecked	(params->allInputs);
	wInputMonths	->setValue		(params->inputMonths);
	wOutputsSingle	->setChecked	(!params->allOutputs);
	wOutputsAll		->setChecked	(params->allOutputs);
	wHidden1		->setValue		(params->hidden1);
	wHidden2		->setValue		(params->hidden2);
	wHidden3		->setValue		(params->hidden3);
	wEqualizeGlobal	->setChecked	(params->equalizeGlobal);
	useWeightDecayChanged ();
	topologyChanged ();
}

void TrainerDlg::defaults () {
	TrainParameters* params = new TrainParameters ();
	paramsToWidgets (params);
	delete params;
}

void TrainerDlg::accept () {
	// Copy data from widgets to the settings
	pParams->trainCycles	= wTrainCycles->value();
	pParams->delta0			= String(wDelta0->text());
	pParams->deltaMax		= String(wDeltaMax->text());
	pParams->useWeightDecay	= wUseWeightDecay->isChecked();
	pParams->weightDecay	= String(wWeightDecay->text());
	pParams->runs			= wRuns->value();
	pParams->monthInputs	= wInputMonth->isChecked();
	pParams->allInputs		= wInputsAll->isChecked();
	pParams->allOutputs		= wOutputsAll->isChecked();
	pParams->inputMonths	= wInputMonths->value();
	pParams->hidden1		= wHidden1->value();
	pParams->hidden2		= wHidden2->value();
	pParams->hidden3		= wHidden3->value();
	pParams->equalizeGlobal	= wEqualizeGlobal->isChecked();
	
	QDialog::accept ();
}

void TrainerDlg::train () {
	accept();
}

void TrainerDlg::useWeightDecayChanged() {
	wWeightDecay->setEnabled (wUseWeightDecay->isChecked());
	wWeightDecayLabel->setEnabled (wUseWeightDecay->isChecked());
}

void TrainerDlg::inputsToggled() {
	if (wInputsSingle->isChecked ())
		wOutputsSingle->setChecked (true);
	else
		if (wOutputsAll->isChecked ())
			wInputsAll->setChecked (true);
	wOutputsAll->setEnabled (wInputsAll->isChecked());
	topologyChanged();
}

void TrainerDlg::topologyChanged () {
	int inputs = wInputMonth->isChecked()*12 + wInputMonths->value()*(wInputsAll->isChecked()? mVariables:1);
	int outputs = wOutputsAll->isChecked()?mVariables:1;
	String hiddens = "-";
	if (wHidden1->value()>0)
		hiddens += String(wHidden1->value()) + "-";
	if (wHidden2->value()>0)
		hiddens += String(wHidden2->value()) + "-";
	if (wHidden3->value()>0)
		hiddens += String(wHidden3->value()) + "-";
	String topology = strformat ("%d%s%d", inputs, (CONSTR) hiddens, outputs);
	wNetworkTopology->setText ((CONSTR) topology);
}
