/***************************************************************************
                          trainprogrdlg.cpp  -  description
                             -------------------
    begin                : Fri Jul 21 2000
    copyright            : (C) 2000 by Marko Grnroos
    email                : magi@iki.fi
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#define format strformat
#include <magic/object.h>
#include <trainer.h>
#undef ASSERT
#undef format

#include <qtimer.h>
#include <pthread.h>
#include <unistd.h>

#include "trainprogrdlg.h"
#include "kauditor.h"
#include "kauditordoc.h"

pthread_t a_thread;
void *thread_result;

void *thread_function(void *arg) {
	TrainProgrDlg* dlg = (TrainProgrDlg*) arg;
	dlg->trainingThread ();
    pthread_exit("Thank you for the CPU time");
}

TrainProgrDlg::TrainProgrDlg(KAuditorApp* parent, const char *name) : QDialog(parent,name,true){
	pDoc = parent->getDocument ();

	initDialog();
	
	mTrainingDone = false;
	mTerminateNow = false;
	mCyclesTrained = 0;
	mVariablesTrained = 0;
	mRunsTrained = 0;
	mTrainCycles = parent->mpParams->trainCycles;
	mRuns = parent->mpParams->runs;
	mVariables = parent->getDocument()->mVariableNames.size;
	
	wCycles->setText("");
	wVariables->setText("");
	wRuns->setText("");
	wVariablesLabel->setEnabled (!parent->mpParams->allOutputs);
	wProgressVariable->setEnabled (!parent->mpParams->allOutputs);
	wVariables->setEnabled (!parent->mpParams->allOutputs);
	wRunsLabel->setEnabled (mRuns>1);
	wProgressRun->setEnabled (mRuns>1);
	wRuns->setEnabled (mRuns>1);
	
	// Initialize thread that does the training
    if (pthread_create(&a_thread, NULL, thread_function, (void *)this)) {
        perror("Thread creation failed");
        exit(EXIT_FAILURE);
    }
	
	// Initialize progress meter update timer
	QTimer *t = new QTimer (this);
	connect( t, SIGNAL(timeout()), SLOT(updateProgressBars()) );
	t->start( 500, FALSE );

	connect (wStop, SIGNAL(clicked()), this, SLOT(stop()));
}

TrainProgrDlg::~TrainProgrDlg(){
}

void TrainProgrDlg::stop() {
	mTerminateNow = true;
    if (pthread_join(a_thread, &thread_result)) {
       perror("Thread join failed at stop()");
        //exit(EXIT_FAILURE);
    }
	reject ();
}

void TrainProgrDlg::updateProgressBars() {
	// Train the network a little

	// Update progress meters
	wProgressRun->setValue(int(float(mCyclesTrained)/mTrainCycles*100));
	wProgressVariable->setValue(int(float(mVariablesTrained)/mVariables*100));
	wProgressTotal->setValue(int(float(mRunsTrained)/mRuns*100));
	
	wCycles->setText((CONSTR) strformat("%d/%d", mCyclesTrained+1, mTrainCycles));
	wVariables->setText((CONSTR) strformat("%d/%d", mVariablesTrained+1, mVariables));
	wRuns->setText((CONSTR) strformat("%d/%d", mRunsTrained+1, mRuns));
	
	if (mTrainingDone) {
	    if (pthread_join(a_thread, &thread_result)) {
	        perror("Thread join failed at updateProgressBars()");
	        //exit(EXIT_FAILURE);
	    }
		accept ();
	}
}

// Ok, here we have an ad hoc callback class for tracking neural training. This
// is pretty useful.
class AuditorTrainingObserver : public TrainingObserver {
	TrainProgrDlg* pDlg;
  public:
  	AuditorTrainingObserver (TrainProgrDlg* dlg) {pDlg = dlg;}
	void	cycleTrained	(const Trainer& trainer, int totalCycles) {
		if (pDlg->trainingObserver (trainer, totalCycles))	
			stopTraining ();
	}
};

void TrainProgrDlg::trainingThread () {
	AuditorTrainingObserver* observer = new AuditorTrainingObserver (this);
	for (mRunsTrained=0; mRunsTrained<mRuns; mRunsTrained++) {
		mVariablesTrained = 0;

		// The document handles all training, testing, and result handling	
		pDoc->trainTestStrategy ("neural", observer);

		// The user-interface thread may have ordered us to terminate	
		if (mTerminateNow)
			break;
	}
	mTrainingDone = true;
}

bool TrainProgrDlg::trainingObserver (const Trainer& trainer, int totalCycles) {
	if (totalCycles<mCyclesTrained)
		mVariablesTrained++;
	mCyclesTrained = totalCycles;
	return mTerminateNow;
}

